/*
Optical flow calculation in OpenCV


Dr. Orion Lawlor, lawlor@alaska.edu, 2015-11-04 (Public Domain)
*/
#include <stdio.h>

#include <opencv2/opencv.hpp>


int main(int argc,char *argv[])
{
	cv::VideoCapture *cap=0;
	
	int argi=1; // command line argument index
	while (argc>=argi+2) { /* keyword-value pairs */
		char *key=argv[argi++];
		char *value=argv[argi++];
		if (0==strcmp(key,"--cam")) cap=new cv::VideoCapture(atoi(value));
		if (0==strcmp(key,"--file")) cap=new cv::VideoCapture(value);
		else printf("Unrecognized command line argument '%s'!\n",key);
	}
	// Initialize capturing live feed from the camera
	if (!cap) cap=new cv::VideoCapture(0);
	
	// Couldn't get a device? Throw an error and quit
	if(!cap->isOpened())
	{
		printf("Could not initialize capturing...\n");
		return -1;
	}
	
	// Will hold a frame captured from the camera
	cv::Mat frameRGB, frameG, frame;
	(*cap)>>frameRGB; // grab first video frame
	cv::cvtColor( frameRGB, frameG, CV_BGR2GRAY ); // to grayscale
	frameG.convertTo(frame,CV_32F); // to float image
	
	cv::Mat derivX=frame.clone();
	cv::Mat derivY=frame.clone();
	cv::Mat debug=frame.clone();
	cv::Mat last_frame=frame.clone();

	// An infinite frame-capturing loop
	while(true)
	{
		(*cap)>>frameRGB; // grab next frame from camera

		// If we couldn't grab a frame... quit
		if(frameRGB.empty())
			break;
		cv::cvtColor( frameRGB, frameG, CV_BGR2GRAY ); // to grayscale
		frameG.convertTo(frame,CV_32F); // to float image
		
		// Compute time derivatives
		cv::Mat dfdt=frame-last_frame;
		frame.copyTo(last_frame);
		dfdt=dfdt*(4.0/256);
		cv::imshow("df/dt", dfdt+0.5);
		
		// Compute spatial derivatives
		cv::Sobel(frame, derivX, CV_32F, 1,0, 3);
		cv::Mat dfdx=derivX*(4.0/(3*3*255));
		cv::Sobel(frame, derivY, CV_32F, 0,1, 3);
		cv::Mat dfdy=derivY*(4.0/(3*3*255));
		cv::imshow("df/dx", dfdx+0.5);
		
		// Loop over pixels, and sum motions
		debug=cv::Scalar(0,0,0);
		double sum_x=0, sum_y=0, sum_X=0, sum_Y=0;
		for (int y=0;y<frame.rows;y++) 
		for (int x=0;x<frame.cols;x++){
			float debugV=0.0;
			float dt=dfdt.at<float>(y,x);
			float dx=dfdx.at<float>(y,x);
			float dy=dfdy.at<float>(y,x);
			if (fabs(dx*dt)>0.1) { // big X motion
				sum_x+=dt/dx; sum_X++; 
			}
			if (fabs(dy*dt)>0.1) { // big Y motion
				sum_y+=dt/dy; sum_Y++; 
			}
			debugV=dx*dt;
			debug.at<float>(y,x)=debugV+0.5;
		}
		cv::imshow("debug",debug);
		
		printf("  %5.2f  %5.2f\n", sum_x/sum_X, sum_y/sum_Y );

		// Wait for a keypress (for up to 1ms)
		int c = cv::waitKey(1);
		if(c!=-1) break;
	}

	return 0;
}
